@@ -6,6 +6,7 @@ import uuid
 from shlex import quote
 from typing import Optional, List, Dict, Tuple
 from cpg_utils.config import get_config
+from cpg_utils.hail_batch import Namespace
 import hailtop.batch as hb
 from analysis_runner.constants import GCLOUD_ACTIVATE_AUTH
 from analysis_runner.git import (
@@ -17,7 +18,7 @@ from analysis_runner.git import (
 )
 
 
-HAIL_VERSION = '0.2.91'
+HAIL_VERSION = '0.2.97'
 DATAPROC_IMAGE = (
     f'australia-southeast1-docker.pkg.dev/analysis-runner/images/'
     f'dataproc:hail-{HAIL_VERSION}'
@@ -28,9 +29,11 @@ DATAPROC_IMAGE = (
 WHEEL = f'gs://cpg-hail-ci/wheels/hail-{HAIL_VERSION}-py3-none-any.whl'
 
 _config = get_config()
+ACCESS_LEVEL = _config['workflow']['access_level']
+NAMESPACE = Namespace.from_access_level(ACCESS_LEVEL).value
+DATASET = _config['workflow']['dataset']
 DATASET_GCP_PROJECT = _config['workflow']['dataset_gcp_project']
 GCLOUD_CONFIG_SET_PROJECT = f'gcloud config set project {DATASET_GCP_PROJECT}'
-DATAPROC_REGION = 'gcloud config set dataproc/region australia-southeast1'
 PYFILES_DIR = '/tmp/pyfiles'
 PYFILES_ZIP = 'pyfiles.zip'
 
@@ -44,6 +47,7 @@ class DataprocCluster:
         self._batch = kwargs.pop('batch')
         self._depends_on = kwargs.pop('depends_on', None)
         self._cluster_name = kwargs.get('cluster_name', None)
+        self._region = kwargs.pop('region')
         self._cluster_id = None
         self._start_job = None
         self._stop_job = None
@@ -54,13 +58,18 @@ class DataprocCluster:
         script: str,
         job_name: Optional[str] = None,
         pyfiles: Optional[List[str]] = None,
+        attributes: Optional[Dict] = None,
+        depends_on: Optional[List] = None,
     ) -> hb.batch.job.Job:
         """
         Create a job that submits the `script` to the cluster
         """
         if self._start_job is None:
             self._start_job, self._cluster_id = _add_start_job(
-                batch=self._batch, **self._startup_params
+                batch=self._batch,
+                attributes=attributes,
+                region=self._region,
+                **self._startup_params,
             )
             if self._depends_on:
                 self._start_job.depends_on(*self._depends_on)
@@ -70,6 +79,8 @@ class DataprocCluster:
                 cluster_id=self._cluster_id,
                 job_name=job_name,
                 cluster_name=self._cluster_name,
+                attributes=attributes,
+                region=self._region,
             )
             self._stop_job.depends_on(self._start_job)
 
@@ -80,9 +91,15 @@ class DataprocCluster:
             pyfiles=pyfiles,
             job_name=job_name,
             cluster_name=self._cluster_name,
+            attributes=attributes,
+            region=self._region,
         )
         job.depends_on(self._start_job)
         self._stop_job.depends_on(job)
+
+        if depends_on:
+            job.depends_on(*depends_on)
+
         return job
 
 
@@ -91,6 +108,7 @@ def setup_dataproc(  # pylint: disable=unused-argument,too-many-arguments
     max_age: str,
     num_workers: int = 2,
     num_secondary_workers: int = 0,
+    region: str = 'australia-southeast1',
     worker_machine_type: Optional[str] = None,  # e.g. 'n1-highmem-8'
     worker_boot_disk_size: Optional[int] = None,  # in GB
     secondary_worker_boot_disk_size: Optional[int] = None,  # in GB
@@ -122,6 +140,7 @@ def hail_dataproc_job(
     script: str,
     pyfiles: Optional[List[str]] = None,
     job_name: Optional[str] = None,
+    attributes: Optional[Dict] = None,
     **kwargs,
 ) -> hb.batch.job.Job:
     """
@@ -129,12 +148,13 @@ def hail_dataproc_job(
     """
     kwargs['job_name'] = job_name
     cluster = setup_dataproc(batch, **kwargs)
-    return cluster.add_job(script, job_name, pyfiles)
+    return cluster.add_job(script, job_name, pyfiles, attributes=attributes)
 
 
 def _add_start_job(  # pylint: disable=too-many-arguments
     batch: hb.Batch,
     max_age: str,
+    region: str,
     num_workers: int = 2,
     num_secondary_workers: int = 0,
     autoscaling_policy: Optional[str] = None,
@@ -149,10 +169,11 @@ def _add_start_job(  # pylint: disable=too-many-arguments
     job_name: Optional[str] = None,
     scopes: Optional[List[str]] = None,
     labels: Optional[Dict[str, str]] = None,
+    attributes: Optional[Dict] = None,
 ) -> Tuple[hb.batch.job.Job, str]:
     """
     Returns a Batch job which starts a Dataproc cluster, and the name of the cluster.
-    The user is respondible for stopping the cluster.
+    The user is responsible for stopping the cluster.
 
     See the `hailctl` tool for information on the keyword parameters.
     """
@@ -171,11 +192,10 @@ def _add_start_job(  # pylint: disable=too-many-arguments
     labels['compute-category'] = 'dataproc'
     labels_formatted = ','.join(f'{key}={value}' for key, value in labels.items())
 
-    start_job = batch.new_job(name=job_name)
+    start_job = batch.new_job(name=job_name, attributes=attributes)
     start_job.image(DATAPROC_IMAGE)
     start_job.command(GCLOUD_ACTIVATE_AUTH)
     start_job.command(GCLOUD_CONFIG_SET_PROJECT)
-    start_job.command(DATAPROC_REGION)
 
     # The spark-env property can be used to set environment variables in jobs that run
     # on the Dataproc cluster. We propagate some currently set environment variables
@@ -186,13 +206,16 @@ def _add_start_job(  # pylint: disable=too-many-arguments
     # Using a space will break some options like --label
     start_job_command = [
         'hailctl dataproc start',
-        f'--service-account=dataproc-{_config["workflow"]["access_level"]}@{DATASET_GCP_PROJECT}.iam.gserviceaccount.com',
+        f'--region={region}',
+        f'--service-account=dataproc-{ACCESS_LEVEL}@{DATASET_GCP_PROJECT}.iam.gserviceaccount.com',
         f'--max-age={max_age}',
         f'--num-workers={num_workers}',
         f'--num-secondary-workers={num_secondary_workers}',
         f'--properties="{",".join(spark_env)}"',
         f'--labels={labels_formatted}',
         f'--wheel={WHEEL}',
+        f'--bucket=cpg-{DATASET}-{NAMESPACE}-tmp',
+        f'--temp-bucket=cpg-{DATASET}-{NAMESPACE}-tmp',
     ]
     if worker_machine_type:
         start_job_command.append(f'--worker-machine-type={worker_machine_type}')
@@ -203,7 +226,7 @@ def _add_start_job(  # pylint: disable=too-many-arguments
             f'--secondary-worker-boot-disk-size={secondary_worker_boot_disk_size}'
         )
     if packages:
-        start_job_command.append(f'--packages={",".join(packages)}')
+        start_job_command.append(f'--packages={quote(",".join(packages))}')
     if init:
         start_job_command.append(f'--init={",".join(init)}')
     if vep:
@@ -226,9 +249,11 @@ def _add_submit_job(
     batch: hb.Batch,
     cluster_id: str,
     script: str,
+    region: str,
     pyfiles: Optional[List[str]] = None,
     job_name: Optional[str] = None,
     cluster_name: Optional[str] = None,
+    attributes: Optional[Dict] = None,
 ) -> hb.batch.job.Job:
     """
     Returns a job that submits a script to the Dataproc cluster
@@ -240,11 +265,10 @@ def _add_submit_job(
     if cluster_name:
         job_name += f' "{cluster_name}"'
 
-    main_job = batch.new_job(name=job_name)
+    main_job = batch.new_job(name=job_name, attributes=attributes)
     main_job.image(DATAPROC_IMAGE)
     main_job.command(GCLOUD_ACTIVATE_AUTH)
     main_job.command(GCLOUD_CONFIG_SET_PROJECT)
-    main_job.command(DATAPROC_REGION)
 
     # Clone the repository to pass scripts to the cluster.
     prepare_git_job(
@@ -256,16 +280,10 @@ def _add_submit_job(
     if cwd:
         main_job.command(f'cd {quote(cwd)}')
 
-    if pyfiles:
-        main_job.command(f'mkdir {PYFILES_DIR}')
-        main_job.command(f'cp -r {" ".join(pyfiles)} {PYFILES_DIR}')
-        main_job.command(f'cd {PYFILES_DIR}')
-        main_job.command(f'zip -r {PYFILES_ZIP} .')
-        main_job.command(f'cd -')
-
     main_job.command(
         f'hailctl dataproc submit '
-        + (f'--pyfiles {PYFILES_DIR}/{PYFILES_ZIP} ' if pyfiles else '')
+        f'--region={region} '
+        + (f'--pyfiles {",".join(pyfiles)} ' if pyfiles else '')
         + f'{cluster_id} {script} '
     )
     return main_job
@@ -274,8 +292,10 @@ def _add_submit_job(
 def _add_stop_job(
     batch: hb.Batch,
     cluster_id: str,
+    region: str,
     job_name: Optional[str] = None,
     cluster_name: Optional[str] = None,
+    attributes: Optional[Dict] = None,
 ) -> hb.batch.job.Job:
     """
     Returns a job that stops the Dataproc cluster specified by `cluster_id`
@@ -285,12 +305,11 @@ def _add_stop_job(
     if cluster_name:
         job_name += f' "{cluster_name}"'
 
-    stop_job = batch.new_job(name=job_name)
+    stop_job = batch.new_job(name=job_name, attributes=attributes)
     stop_job.always_run()  # Always clean up.
     stop_job.image(DATAPROC_IMAGE)
     stop_job.command(GCLOUD_ACTIVATE_AUTH)
     stop_job.command(GCLOUD_CONFIG_SET_PROJECT)
-    stop_job.command(DATAPROC_REGION)
-    stop_job.command(f'hailctl dataproc stop {cluster_id}')
+    stop_job.command(f'hailctl dataproc stop --region={region} {cluster_id}')
 
     return stop_job
